// src/backend/holt_graph.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef HOLT_GRAPH_HH
#define HOLT_GRAPH_HH 1

namespace sgpem
{
  class Resource;
  class Thread;
  class Request;
}

#include "history_observer.hh"

#include <set>
#include <vector>

namespace sgpem
{
  class HoltGraph;

  class SG_DLLEXPORT HoltGraph : public HistoryObserver
  {
  public:
    typedef std::vector<const Resource*> Resources;
    typedef std::vector<const Thread*> Threads;
    typedef std::vector<const Request*> Requests;

    HoltGraph();

    virtual void update(const History& changed_history);

    Resources get_resources() const;
    Threads get_active_threads() const;
    Requests get_active_requests() const;

  private:
    std::set<const Resource*> _resources;
    std::set<const Thread*> _active_threads;
    Requests _active_requests;
  }
  ; // class HistoryObserver

}//~ namespace sgpem

#endif //HOLT_GRAPH_HH

